function applyDecs2203RFactory() {
    function createAddInitializerMethod(initializers, decoratorFinishedRef) {
        return function addInitializer(initializer) {
            assertNotFinished(decoratorFinishedRef, "addInitializer");
            assertCallable(initializer, "An initializer");
            initializers.push(initializer);
        };
    }
    function memberDec(dec, name, desc, initializers, kind, isStatic, isPrivate, metadata, value) {
        var kindStr;
        switch(kind){
            case 1:
                kindStr = "accessor";
                break;
            case 2:
                kindStr = "method";
                break;
            case 3:
                kindStr = "getter";
                break;
            case 4:
                kindStr = "setter";
                break;
            default:
                kindStr = "field";
        }
        var ctx = {
            kind: kindStr,
            name: isPrivate ? "#" + name : name,
            static: isStatic,
            private: isPrivate,
            metadata: metadata
        };
        var decoratorFinishedRef = {
            v: false
        };
        ctx.addInitializer = createAddInitializerMethod(initializers, decoratorFinishedRef);
        var get, set;
        if (kind === 0) {
            if (isPrivate) {
                get = desc.get;
                set = desc.set;
            } else {
                get = function() {
                    return this[name];
                };
                set = function(v) {
                    this[name] = v;
                };
            }
        } else if (kind === 2) {
            get = function() {
                return desc.value;
            };
        } else {
            if (kind === 1 || kind === 3) {
                get = function() {
                    return desc.get.call(this);
                };
            }
            if (kind === 1 || kind === 4) {
                set = function(v) {
                    desc.set.call(this, v);
                };
            }
        }
        ctx.access = get && set ? {
            get: get,
            set: set
        } : get ? {
            get: get
        } : {
            set: set
        };
        try {
            return dec(value, ctx);
        } finally{
            decoratorFinishedRef.v = true;
        }
    }
    function assertNotFinished(decoratorFinishedRef, fnName) {
        if (decoratorFinishedRef.v) {
            throw new Error("attempted to call " + fnName + " after decoration was finished");
        }
    }
    function assertCallable(fn, hint) {
        if (typeof fn !== "function") {
            throw new TypeError(hint + " must be a function");
        }
    }
    function assertValidReturnValue(kind, value) {
        var type = typeof value;
        if (kind === 1) {
            if (type !== "object" || value === null) {
                throw new TypeError("accessor decorators must return an object with get, set, or init properties or void 0");
            }
            if (value.get !== undefined) {
                assertCallable(value.get, "accessor.get");
            }
            if (value.set !== undefined) {
                assertCallable(value.set, "accessor.set");
            }
            if (value.init !== undefined) {
                assertCallable(value.init, "accessor.init");
            }
        } else if (type !== "function") {
            var hint;
            if (kind === 0) {
                hint = "field";
            } else if (kind === 10) {
                hint = "class";
            } else {
                hint = "method";
            }
            throw new TypeError(hint + " decorators must return a function or void 0");
        }
    }
    function applyMemberDec(ret, base, decInfo, name, kind, isStatic, isPrivate, initializers, metadata) {
        var decs = decInfo[0];
        var desc, init, value;
        if (isPrivate) {
            if (kind === 0 || kind === 1) {
                desc = {
                    get: decInfo[3],
                    set: decInfo[4]
                };
            } else if (kind === 3) {
                desc = {
                    get: decInfo[3]
                };
            } else if (kind === 4) {
                desc = {
                    set: decInfo[3]
                };
            } else {
                desc = {
                    value: decInfo[3]
                };
            }
        } else if (kind !== 0) {
            desc = Object.getOwnPropertyDescriptor(base, name);
        }
        if (kind === 1) {
            value = {
                get: desc.get,
                set: desc.set
            };
        } else if (kind === 2) {
            value = desc.value;
        } else if (kind === 3) {
            value = desc.get;
        } else if (kind === 4) {
            value = desc.set;
        }
        var newValue, get, set;
        if (typeof decs === "function") {
            newValue = memberDec(decs, name, desc, initializers, kind, isStatic, isPrivate, metadata, value);
            if (newValue !== void 0) {
                assertValidReturnValue(kind, newValue);
                if (kind === 0) {
                    init = newValue;
                } else if (kind === 1) {
                    init = newValue.init;
                    get = newValue.get || value.get;
                    set = newValue.set || value.set;
                    value = {
                        get: get,
                        set: set
                    };
                } else {
                    value = newValue;
                }
            }
        } else {
            for(var i = decs.length - 1; i >= 0; i--){
                var dec = decs[i];
                newValue = memberDec(dec, name, desc, initializers, kind, isStatic, isPrivate, metadata, value);
                if (newValue !== void 0) {
                    assertValidReturnValue(kind, newValue);
                    var newInit;
                    if (kind === 0) {
                        newInit = newValue;
                    } else if (kind === 1) {
                        newInit = newValue.init;
                        get = newValue.get || value.get;
                        set = newValue.set || value.set;
                        value = {
                            get: get,
                            set: set
                        };
                    } else {
                        value = newValue;
                    }
                    if (newInit !== void 0) {
                        if (init === void 0) {
                            init = newInit;
                        } else if (typeof init === "function") {
                            init = [
                                init,
                                newInit
                            ];
                        } else {
                            init.push(newInit);
                        }
                    }
                }
            }
        }
        if (kind === 0 || kind === 1) {
            if (init === void 0) {
                init = function(instance, init) {
                    return init;
                };
            } else if (typeof init !== "function") {
                var ownInitializers = init;
                init = function(instance, init) {
                    var value = init;
                    for(var i = 0; i < ownInitializers.length; i++){
                        value = ownInitializers[i].call(instance, value);
                    }
                    return value;
                };
            } else {
                var originalInitializer = init;
                init = function(instance, init) {
                    return originalInitializer.call(instance, init);
                };
            }
            ret.push(init);
        }
        if (kind !== 0) {
            if (kind === 1) {
                desc.get = value.get;
                desc.set = value.set;
            } else if (kind === 2) {
                desc.value = value;
            } else if (kind === 3) {
                desc.get = value;
            } else if (kind === 4) {
                desc.set = value;
            }
            if (isPrivate) {
                if (kind === 1) {
                    ret.push(function(instance, args) {
                        return value.get.call(instance, args);
                    });
                    ret.push(function(instance, args) {
                        return value.set.call(instance, args);
                    });
                } else if (kind === 2) {
                    ret.push(value);
                } else {
                    ret.push(function(instance, args) {
                        return value.call(instance, args);
                    });
                }
            } else {
                Object.defineProperty(base, name, desc);
            }
        }
    }
    function applyMemberDecs(Class, decInfos, metadata) {
        var ret = [];
        var protoInitializers;
        var staticInitializers;
        var existingProtoNonFields = new Map();
        var existingStaticNonFields = new Map();
        for(var i = 0; i < decInfos.length; i++){
            var decInfo = decInfos[i];
            if (!Array.isArray(decInfo)) continue;
            var kind = decInfo[1];
            var name = decInfo[2];
            var isPrivate = decInfo.length > 3;
            var isStatic = kind >= 5;
            var base;
            var initializers;
            if (isStatic) {
                base = Class;
                kind = kind - 5;
                staticInitializers = staticInitializers || [];
                initializers = staticInitializers;
            } else {
                base = Class.prototype;
                protoInitializers = protoInitializers || [];
                initializers = protoInitializers;
            }
            if (kind !== 0 && !isPrivate) {
                var existingNonFields = isStatic ? existingStaticNonFields : existingProtoNonFields;
                var existingKind = existingNonFields.get(name) || 0;
                if (existingKind === true || existingKind === 3 && kind !== 4 || existingKind === 4 && kind !== 3) {
                    throw new Error("Attempted to decorate a public method/accessor that has the same name as a previously decorated public method/accessor. This is not currently supported by the decorators plugin. Property name was: " + name);
                } else if (!existingKind && kind > 2) {
                    existingNonFields.set(name, kind);
                } else {
                    existingNonFields.set(name, true);
                }
            }
            applyMemberDec(ret, base, decInfo, name, kind, isStatic, isPrivate, initializers, metadata);
        }
        pushInitializers(ret, protoInitializers);
        pushInitializers(ret, staticInitializers);
        return ret;
    }
    function pushInitializers(ret, initializers) {
        if (initializers) {
            ret.push(function(instance) {
                for(var i = 0; i < initializers.length; i++){
                    initializers[i].call(instance);
                }
                return instance;
            });
        }
    }
    function applyClassDecs(targetClass, classDecs, metadata) {
        if (classDecs.length > 0) {
            var initializers = [];
            var newClass = targetClass;
            var name = targetClass.name;
            for(var i = classDecs.length - 1; i >= 0; i--){
                var decoratorFinishedRef = {
                    v: false
                };
                try {
                    var nextNewClass = classDecs[i](newClass, {
                        kind: "class",
                        name: name,
                        addInitializer: createAddInitializerMethod(initializers, decoratorFinishedRef),
                        metadata
                    });
                } finally{
                    decoratorFinishedRef.v = true;
                }
                if (nextNewClass !== undefined) {
                    assertValidReturnValue(10, nextNewClass);
                    newClass = nextNewClass;
                }
            }
            return [
                defineMetadata(newClass, metadata),
                function() {
                    for(var i = 0; i < initializers.length; i++){
                        initializers[i].call(newClass);
                    }
                }
            ];
        }
    }
    function defineMetadata(Class, metadata) {
        return Object.defineProperty(Class, Symbol.metadata || Symbol.for("Symbol.metadata"), {
            configurable: true,
            enumerable: true,
            value: metadata
        });
    }
    return function applyDecs2203R(targetClass, memberDecs, classDecs, parentClass) {
        if (parentClass !== void 0) {
            var parentMetadata = parentClass[Symbol.metadata || Symbol.for("Symbol.metadata")];
        }
        var metadata = Object.create(parentMetadata === void 0 ? null : parentMetadata);
        var e = applyMemberDecs(targetClass, memberDecs, metadata);
        if (!classDecs.length) defineMetadata(targetClass, metadata);
        return {
            e: e,
            get c () {
                return applyClassDecs(targetClass, classDecs, metadata);
            }
        };
    };
}
function _apply_decs_2203_r(targetClass, memberDecs, classDecs, parentClass) {
    return (_apply_decs_2203_r = applyDecs2203RFactory())(targetClass, memberDecs, classDecs, parentClass);
}
var _initProto;
import { GoogleGenerativeAI } from "@google/generative-ai";
import { wrapLLMEvent } from "@llamaindex/core/decorator";
import { ToolCallLLM } from "@llamaindex/core/llms";
import { streamConverter } from "@llamaindex/core/utils";
import { getEnv, randomUUID } from "@llamaindex/env";
import { GEMINI_BACKENDS, GEMINI_MODEL } from "./types.js";
import { DEFAULT_SAFETY_SETTINGS, GeminiHelper, getChatContext, getPartsText, mapBaseToolToGeminiFunctionDeclaration } from "./utils.js";
export const GEMINI_MODEL_INFO_MAP = {
    [GEMINI_MODEL.GEMINI_PRO]: {
        contextWindow: 30720
    },
    [GEMINI_MODEL.GEMINI_PRO_VISION]: {
        contextWindow: 12288
    },
    // multi-modal/multi turn
    [GEMINI_MODEL.GEMINI_PRO_LATEST]: {
        contextWindow: 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_FLASH_LATEST]: {
        contextWindow: 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5_PRO_PREVIEW]: {
        contextWindow: 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5_FLASH_PREVIEW]: {
        contextWindow: 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5]: {
        contextWindow: 2 * 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5_FLASH]: {
        contextWindow: 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5_LATEST]: {
        contextWindow: 2 * 10 ** 6
    },
    [GEMINI_MODEL.GEMINI_PRO_1_5_FLASH_LATEST]: {
        contextWindow: 10 ** 6
    }
};
const SUPPORT_TOOL_CALL_MODELS = [
    GEMINI_MODEL.GEMINI_PRO,
    GEMINI_MODEL.GEMINI_PRO_VISION,
    GEMINI_MODEL.GEMINI_PRO_1_5_PRO_PREVIEW,
    GEMINI_MODEL.GEMINI_PRO_1_5_FLASH_PREVIEW,
    GEMINI_MODEL.GEMINI_PRO_1_5,
    GEMINI_MODEL.GEMINI_PRO_1_5_FLASH,
    GEMINI_MODEL.GEMINI_PRO_LATEST,
    GEMINI_MODEL.GEMINI_PRO_FLASH_LATEST,
    GEMINI_MODEL.GEMINI_PRO_1_5_LATEST,
    GEMINI_MODEL.GEMINI_PRO_1_5_FLASH_LATEST
];
const DEFAULT_GEMINI_PARAMS = {
    model: GEMINI_MODEL.GEMINI_PRO,
    temperature: 0.1,
    topP: 1,
    maxTokens: undefined
};
/**
 * Gemini Session to manage the connection to the Gemini API
 */ export class GeminiSession {
    gemini;
    constructor(options){
        if (!options.apiKey) {
            options.apiKey = getEnv("GOOGLE_API_KEY");
        }
        if (!options.apiKey) {
            throw new Error("Set Google API Key in GOOGLE_API_KEY env variable");
        }
        this.gemini = new GoogleGenerativeAI(options.apiKey);
    }
    getGenerativeModel(metadata) {
        return this.gemini.getGenerativeModel({
            safetySettings: DEFAULT_SAFETY_SETTINGS,
            ...metadata
        });
    }
    getResponseText(response) {
        return response.text();
    }
    getToolsFromResponse(response) {
        return response.functionCalls()?.map((call)=>({
                name: call.name,
                input: call.args,
                id: randomUUID()
            }));
    }
    async *getChatStream(result) {
        yield* streamConverter(result.stream, (response)=>{
            const tools = this.getToolsFromResponse(response);
            const options = tools?.length ? {
                toolCall: tools
            } : {};
            return {
                delta: this.getResponseText(response),
                raw: response,
                options
            };
        });
    }
    getCompletionStream(result) {
        return streamConverter(result.stream, (response)=>({
                text: this.getResponseText(response),
                raw: response
            }));
    }
}
/**
 * Gemini Session Store to manage the current Gemini sessions
 */ export class GeminiSessionStore {
    static sessions = [];
    static getSessionId(options) {
        if (options.backend === GEMINI_BACKENDS.GOOGLE) {
            return options?.apiKey ?? "";
        }
        return "";
    }
    static sessionMatched(o1, o2) {
        return GeminiSessionStore.getSessionId(o1) === GeminiSessionStore.getSessionId(o2);
    }
    static get(options = {
        backend: GEMINI_BACKENDS.GOOGLE
    }) {
        let session = this.sessions.find((session)=>this.sessionMatched(session.options, options))?.session;
        if (session) return session;
        if (options.backend === GEMINI_BACKENDS.VERTEX) {
            throw Error("No Session");
        } else {
            session = new GeminiSession(options);
        }
        this.sessions.push({
            session,
            options
        });
        return session;
    }
}
/**
 * ToolCallLLM for Gemini
 */ export class Gemini extends ToolCallLLM {
    static{
        ({ e: [_initProto] } = _apply_decs_2203_r(this, [
            [
                wrapLLMEvent,
                2,
                "chat"
            ]
        ], []));
    }
    model;
    temperature;
    topP;
    maxTokens;
    session;
    constructor(init){
        super(), _initProto(this);
        this.model = init?.model ?? GEMINI_MODEL.GEMINI_PRO;
        this.temperature = init?.temperature ?? 0.1;
        this.topP = init?.topP ?? 1;
        this.maxTokens = init?.maxTokens ?? undefined;
        this.session = init?.session ?? GeminiSessionStore.get();
    }
    get supportToolCall() {
        return SUPPORT_TOOL_CALL_MODELS.includes(this.model);
    }
    get metadata() {
        return {
            model: this.model,
            temperature: this.temperature,
            topP: this.topP,
            maxTokens: this.maxTokens,
            contextWindow: GEMINI_MODEL_INFO_MAP[this.model].contextWindow,
            tokenizer: undefined
        };
    }
    async nonStreamChat(params) {
        const context = getChatContext(params);
        const client = this.session.getGenerativeModel(this.metadata);
        const chat = client.startChat(params.tools ? {
            history: context.history,
            tools: [
                {
                    functionDeclarations: params.tools.map(mapBaseToolToGeminiFunctionDeclaration)
                }
            ],
            safetySettings: DEFAULT_SAFETY_SETTINGS
        } : {
            history: context.history,
            safetySettings: DEFAULT_SAFETY_SETTINGS
        });
        const { response } = await chat.sendMessage(context.message);
        const topCandidate = response.candidates[0];
        const tools = this.session.getToolsFromResponse(response);
        const options = tools?.length ? {
            toolCall: tools
        } : {};
        return {
            raw: response,
            message: {
                content: this.session.getResponseText(response),
                role: GeminiHelper.ROLES_FROM_GEMINI[topCandidate.content.role],
                options
            }
        };
    }
    async *streamChat(params) {
        const context = getChatContext(params);
        const client = this.session.getGenerativeModel(this.metadata);
        const chat = client.startChat(params.tools ? {
            history: context.history,
            tools: [
                {
                    functionDeclarations: params.tools.map(mapBaseToolToGeminiFunctionDeclaration)
                }
            ],
            safetySettings: DEFAULT_SAFETY_SETTINGS
        } : {
            history: context.history,
            safetySettings: DEFAULT_SAFETY_SETTINGS
        });
        const result = await chat.sendMessageStream(context.message);
        yield* this.session.getChatStream(result);
    }
    async chat(params) {
        if (params.stream) return this.streamChat(params);
        return this.nonStreamChat(params);
    }
    async complete(params) {
        const { prompt, stream } = params;
        const client = this.session.getGenerativeModel(this.metadata);
        if (stream) {
            const result = await client.generateContentStream(getPartsText(GeminiHelper.messageContentToGeminiParts({
                content: prompt
            })));
            return this.session.getCompletionStream(result);
        }
        const result = await client.generateContent(getPartsText(GeminiHelper.messageContentToGeminiParts({
            content: prompt
        })));
        return {
            text: this.session.getResponseText(result.response),
            raw: result.response
        };
    }
}
