import { AzureDynamicSessionTool } from "./AzureDynamicSessionTool.node.js";
import { WikipediaTool } from "./WikipediaTool.js";
(function(ToolsFactory) {
    (function(Tools) {
        Tools["Wikipedia"] = "wikipedia.WikipediaToolSpec";
        Tools["AzureCodeInterpreter"] = "azure_code_interpreter.AzureCodeInterpreterToolSpec";
    })(ToolsFactory.Tools || (ToolsFactory.Tools = {}));
    async function createTool(key, ...params) {
        if (key === "wikipedia.WikipediaToolSpec") {
            return new WikipediaTool(...params);
        }
        if (key === "azure_code_interpreter.AzureCodeInterpreterToolSpec") {
            return new AzureDynamicSessionTool(...params);
        }
        throw new Error(`Sorry! Tool ${key} is not supported yet. Options: ${params}`);
    }
    ToolsFactory.createTool = createTool;
    async function createTools(record) {
        const tools = [];
        for(const key in record){
            const params = record[key];
            tools.push(await createTool(key, // @ts-expect-error allow array or single parameter
            Array.isArray(params) ? params : [
                params
            ]));
        }
        return tools;
    }
    ToolsFactory.createTools = createTools;
})(ToolsFactory || (ToolsFactory = {}));
export var ToolsFactory;
