import type { StoredValue } from "@llamaindex/core/schema";
import { BaseKVStore } from "@llamaindex/core/storage/kv-store";
import type pg from "pg";
export type DataType = Record<string, Record<string, StoredValue>>;
export type PostgresKVStoreBaseConfig = {
    schemaName?: string | undefined;
    tableName?: string | undefined;
};
export type PostgresKVStoreClientConfig = {
    /**
     * Client configuration options for the pg client.
     *
     * {@link https://node-postgres.com/apis/client#new-client PostgresSQL Client API}
     */
    clientConfig?: pg.ClientConfig | undefined;
} | {
    /**
     * A pg client or pool client instance.
     * If provided, make sure it is not connected to the database yet, or it will throw an error.
     */
    shouldConnect?: boolean | undefined;
    client?: pg.Client | pg.PoolClient;
};
export type PostgresKVStoreConfig = PostgresKVStoreBaseConfig & PostgresKVStoreClientConfig;
export declare class PostgresKVStore extends BaseKVStore {
    private schemaName;
    private tableName;
    private isDBConnected;
    private clientConfig;
    private db?;
    constructor(config?: PostgresKVStoreConfig);
    private getDb;
    private checkSchema;
    client(): Promise<pg.ClientBase>;
    put(key: string, val: StoredValue, collection?: string): Promise<void>;
    get(key: string, collection?: string): Promise<any>;
    getAll(collection?: string): Promise<DataType>;
    delete(key: string, collection?: string): Promise<boolean>;
}
